@@ -1,5 +1,3 @@
-context("block list")
-
 fp_bold <- shortcuts$fp_bold()
 
 test_that("block_list structure", {
@@ -21,13 +19,4 @@ test_that("block_list structure", {
   expect_is(bl[[2]]$chunks[[2]], "cot")
 })
 
-test_that("print block_list", {
-  img.file <- file.path( R.home("doc"), "html", "logo.jpg" )
-  bl <- block_list(
-    fpar( ftext("hello", fp_bold),
-          external_img(src = img.file, height = 1.06, width = 1.39)
-          )
-    )
-  expect_output(print(bl), "{text:{hello}}{image:", fixed = TRUE)
-})
 
